@@ -66,8 +66,8 @@ from nuitka.nodes.StatementNodes import (
     StatementsSequence,
 )
 from nuitka.nodes.VariableAssignNodes import makeStatementAssignmentVariable
-from nuitka.nodes.VariableDelNodes import StatementReleaseVariable
 from nuitka.nodes.VariableRefNodes import ExpressionTempVariableRef
+from nuitka.nodes.VariableReleaseNodes import makeStatementReleaseVariable
 from nuitka.nodes.YieldNodes import (
     ExpressionYield,
     ExpressionYieldFromWaitable,
@@ -240,6 +240,7 @@ def buildGeneratorExpressionNode(provider, node, source_ref):
 
     code_object = CodeObjectSpec(
         co_name="<genexpr>",
+        co_qualname=provider.getChildQualname("<genexpr>"),
         co_kind="Generator",
         co_varnames=(".0",),
         co_freevars=(),
@@ -309,7 +310,7 @@ def buildGeneratorExpressionNode(provider, node, source_ref):
                     ),
                     source_ref=source_ref,
                 ),
-                final=StatementReleaseVariable(
+                final=makeStatementReleaseVariable(
                     variable=iter_tmp, source_ref=source_ref
                 ),
                 source_ref=source_ref,
@@ -567,7 +568,7 @@ def _buildContractionBodyNode(
 
         if tmp_iter_variable is not None:
             nested_statements.append(
-                StatementReleaseVariable(
+                makeStatementReleaseVariable(
                     variable=tmp_iter_variable, source_ref=source_ref
                 )
             )
@@ -580,7 +581,7 @@ def _buildContractionBodyNode(
     statements = mergeStatements(statements)
 
     release_statements = [
-        StatementReleaseVariable(variable=tmp_variable, source_ref=source_ref)
+        makeStatementReleaseVariable(variable=tmp_variable, source_ref=source_ref)
         for tmp_variable in tmp_variables
     ]
 
@@ -652,6 +653,7 @@ def _buildContractionNode(provider, node, name, emit_class, start_value, source_
 
         code_object = CodeObjectSpec(
             co_name=name,
+            co_qualname=provider.getChildQualname(name),
             co_kind="Function",
             co_varnames=(),
             co_freevars=(),
